// This file is safe to edit. Once it exists it will not be overwritten

package restapi

import (
	"crypto/tls"
	"net/http"
	"strings"
	"time"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/runtime"
	"github.com/jmoiron/sqlx"
	"github.com/justinas/alice"
	"github.com/rs/zerolog"
	"github.com/rs/zerolog/hlog"

	"orus.io/orus-io/rednerd/auth"
	"orus.io/orus-io/rednerd/rendering"
	"orus.io/orus-io/rednerd/restapi/handlers"
	"orus.io/orus-io/rednerd/restapi/operations"
)

//go:generate swagger generate server --target ../../rednerd --name Rednerd --spec ../swagger.yaml --principal models.Principal

// APIConfig holds the configuration options for the API.
type APIConfig struct {
	Log zerolog.Logger
	DB  *sqlx.DB

	RenderingEngineRegistry         *rendering.EngineRegistry
	TemplateRenderingEngineRegistry *rendering.TemplateEngineRegistry

	TokenOptions *auth.TokenOptions
}

func configureFlags(api *operations.RednerdAPI) {
}

// ConfigureAPI setup the api.
func (config APIConfig) ConfigureAPI(s *Server, api *operations.RednerdAPI) http.Handler {
	// configure the api here
	api.ServeError = errors.ServeError

	// Set your custom logger if needed. Default one is log.Printf
	// Expected interface func(string, ...interface{})
	//
	// Example:
	// api.Logger = log.Printf

	api.JSONConsumer = runtime.JSONConsumer()

	api.JSONProducer = runtime.JSONProducer()

	auth := auth.NewAuth(*config.TokenOptions, config.DB, config.Log)

	// Applies when the "Authorization" header is set
	api.TokenAuth = auth.AuthFromAuthorization
	// Applies when the "Cookie" header is set
	api.CookieAuth = auth.AuthFromCookie
	api.ApikeyAuth = auth.AuthFromAPIKey

	api.AuthAuthenticateHandler = handlers.NewAuthAuthenticateHandler(auth)
	api.AuthUserinfoHandler = handlers.NewAuthUserinfoHandler(config.DB)
	api.AuthRefreshTokenHandler = handlers.NewAuthRefreshTokenHandler(config.DB, auth)

	api.UserManagementUserListHandler = handlers.NewUserListHandler(config.DB)
	api.UserManagementUserAddHandler = handlers.NewUserAddHandler(config.DB)
	api.UserManagementUserGetHandler = handlers.NewUserGetHandler(config.DB)
	api.UserManagementUserUpdateHandler = handlers.NewUserUpdateHandler(config.DB)
	api.UserManagementUserDeleteHandler = handlers.NewUserDeleteHandler(config.DB)

	api.UserManagementUserPasswordSetHandler = handlers.NewUserPasswordSetHandler(config.DB)
	api.UserManagementUserPasswordDeleteHandler = handlers.NewUserPasswordDeleteHandler(config.DB)

	api.UserManagementUserApikeyAddHandler = handlers.NewUserApikeyAddHandler(config.DB)
	api.UserManagementUserApikeyListHandler = handlers.NewUserApikeyListHandler(config.DB)
	api.UserManagementUserApikeyDeleteHandler = handlers.NewUserApikeyDeleteHandler(config.DB)

	api.TemplateManagementAccountTemplateListHandler = handlers.NewAccountTemplateListHandler(config.DB)
	api.TemplateManagementAccountTemplateAddHandler = handlers.NewAccountTemplateAddHandler(config.DB)
	api.TemplateManagementAccountTemplateGetHandler = handlers.NewAccountTemplateGetHandler(config.DB)
	api.TemplateManagementAccountTemplateUpdateHandler = handlers.NewAccountTemplateUpdateHandler(config.DB)
	api.TemplateManagementAccountTemplateDeleteHandler = handlers.NewAccountTemplateDeleteHandler(config.DB)

	api.TemplateManagementTemplateVarlistHandler = handlers.NewTemplateVarlistHandler(
		config.DB, config.TemplateRenderingEngineRegistry)
	api.RenderingRenderHandler = handlers.NewRenderHandler(
		config.DB,
		config.RenderingEngineRegistry,
		config.TemplateRenderingEngineRegistry,
	)

	api.PreServerShutdown = func() {}

	api.ServerShutdown = func() {
		auth.Close()
	}

	handler := api.Serve(setupMiddlewares)

	if s.Prometheus {
		api.PrometheusInstrumentHandlers()
	}

	return config.setupGlobalMiddleware(auth, s.Prometheus)(handler)
}

// The TLS configuration before HTTPS server starts.
func configureTLS(tlsConfig *tls.Config) {
	// Make all necessary changes to the TLS configuration here.
}

// As soon as server is initialized but not run yet, this function will be called.
// If you need to modify a config, store server instance to stop it individually later, this is the place.
// This function can be called multiple times, depending on the number of serving schemes.
// scheme value will be set accordingly: "http", "https" or "unix".
func configureServer(s *http.Server, scheme, addr string) {
}

// The middleware configuration is for the handler executors. These do not apply to the swagger.json document.
// The middleware executes after routing but before authentication, binding and validation.
func setupMiddlewares(handler http.Handler) http.Handler {
	return handler
}

// The middleware configuration happens before anything, this middleware also applies to serving the swagger.json document.
// So this is a good place to plug in a panic handling middleware, logging and metrics.
func (config *APIConfig) setupGlobalMiddleware(auth *auth.Auth, prometheus bool) func(http.Handler) http.Handler {
	stack := alice.New(
		hlog.NewHandler(config.Log),
		CatchPanics,
		hlog.RemoteAddrHandler("ip"),
		hlog.UserAgentHandler("user_agent"),
		hlog.RefererHandler("referer"),
		hlog.RequestIDHandler("req_id", "Request-Id"),
		hlog.AccessHandler(func(r *http.Request, status, size int, duration time.Duration) {
			log := hlog.FromRequest(r)
			var event *zerolog.Event
			switch {
			case status >= http.StatusInternalServerError:
				event = log.Error()
			case status >= http.StatusBadRequest:
				event = log.Info()
			default:
				event = log.Info()
			}
			event.
				Dict("request", zerolog.Dict().
					Str("method", r.Method).
					Str("url", r.URL.String())).
				Int("status", status).
				Int("size", size).
				Dur("duration", duration).
				Str("cookies", strings.Join(r.Header.Values("Cookie"), ",")).
				Msg(http.StatusText(status))
		}),
		AuthMiddleware(auth, config.DB, config.Log),
		Redoc(RedocOpts{Title: "RednerD API Documentation"}),
		EngineDoc(
			"/docs/",
			rendering.DefaultEngineRegistry,
			rendering.DefaultTemplateEngineRegistry,
			config.Log,
		),
	)
	if prometheus {
		stack = stack.Append(Prometheus("/metrics"))
	}

	return stack.Then
}
