package restapi

import (
	"bytes"
	"html/template"
	"net/http"
	"strings"

	"github.com/rs/zerolog"
	"github.com/yuin/goldmark"

	"orus.io/orus-io/rednerd/rendering"
)

// EngineDoc publish the engines documentations on the given path.
func EngineDoc(
	path string,
	registry *rendering.EngineRegistry,
	templateRegistry *rendering.TemplateEngineRegistry,
	log zerolog.Logger,
) func(http.Handler) http.Handler {
	return func(next http.Handler) http.Handler {
		if next == nil {
			panic("EngineDoc: next must be non-nil")
		}

		return http.HandlerFunc(func(rw http.ResponseWriter, r *http.Request) {
			if strings.HasPrefix(r.URL.Path, path) {
				if strings.HasSuffix(r.URL.Path, "/engines") {
					if err := engineListDocPageTmpl.Execute(rw, map[string]interface{}{
						"Engines":         registry.Engines,
						"TemplateEngines": templateRegistry.TemplateEngines,
					}); err != nil {
						log.Err(err).Msg("failed to render the engines documentation page")
						rw.WriteHeader(http.StatusInternalServerError)
						if _, err := rw.Write([]byte(err.Error())); err != nil {
							log.Err(err).Msg("failed to write the error response")
						}
					}

					return
				}
			}
			next.ServeHTTP(rw, r)
		})
	}
}

var engineListDocPageTmpl = template.Must(template.New("engine-doc").
	Funcs(template.FuncMap{
		"goldmark": func(src string) (template.HTML, error) {
			var buf bytes.Buffer
			if err := goldmark.Convert([]byte(src), &buf); err != nil {
				return "", err
			}

			return template.HTML(buf.String()), nil // nolint:gosec
		},
	}).Parse(
	`
<head>
<title>RednerD Engines</title>
</head>
<body>
<h1>RednerD Engines</h1>
<h2>Template engines</h2>
{{ range $name, $engine := .TemplateEngines }}
<h3>{{ $name }} ({{ $engine.Info.Type }})</h3>
<h4>language</h4> 
{{ $engine.Info.Language }}

<h4>Description</h4> 
{{ goldmark $engine.Info.Description }} 

{{ if $engine.Info.Metadata }}
<h4>Metadata</h4> 
{{ range $engine.Info.Metadata }}
<dl>
	<dt>{{ .Name }}</dt>
	<dd>{{ goldmark .Description }}</dd>
</dl>
{{ end }}
{{ end }}

{{ end }}
<h2>Rendering engines</h2>
{{ range $name, $engine := .Engines }}
<h3>{{ $name }} ({{ $engine.Info.Type }})</h3>
<h4>Transitions</h4>
{{ range $engine.Info.Transitions }}
<ul>
	<li>{{ .FromType }} -> {{ .ToType }} ({{ .Priority }})</li>
</ul>
{{ end }}
<h4>Description</h4>
{{ goldmark $engine.Info.Description }}
{{ if $engine.Info.Metadata }}
<h4>Metadata</h4> 
{{ range $engine.Info.Metadata }}
<dl>
	<dt>{{ .Name }}</dt>
	<dd>{{ goldmark .Description }}</dd>
</dl>
{{ end }}
{{ end }}
{{ end }}
</body>
	`,
))
