package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/template_management"
)

// NewAccountTemplateListHandler returns a AccountTemplateListHandler.
func NewAccountTemplateListHandler(db *sqlx.DB) *AccountTemplateListHandler {
	return &AccountTemplateListHandler{
		db: db,
	}
}

// AccountTemplateListHandler handles GET /user.
type AccountTemplateListHandler struct {
	db *sqlx.DB
}

func (h *AccountTemplateListHandler) handle(
	ctx context.Context, account string,
) ([]*models.Template, error) {
	log := zerolog.Ctx(ctx)

	var templatedbList []models.TemplateDB

	sql := database.NewSQLHelper(ctx, h.db, *log)

	if err := sql.Select(&templatedbList,
		squirrel.Select(models.TemplateDBColumns...).
			From(models.TemplateDBTable).
			Where(
				squirrel.Eq{
					models.TemplateDBAccountColumn: account,
				}),
	); err != nil {
		return nil, err
	}
	list := make([]*models.Template, 0, len(templatedbList))
	for i := range templatedbList {
		templatedbList[i].FromDB()
		templatedbList[i].ForceBase64()

		list = append(list, &templatedbList[i].Template)
	}

	return list, nil
}

// Handle handles the incoming request.
func (h *AccountTemplateListHandler) Handle(
	params template_management.AccountTemplateListParams, principal *models.Principal,
) middleware.Responder {
	if principal.IsAdmin {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(&models.Error{
				Message: "Admin cannot manage templates. Please use a non-admin user",
			})
	}
	if principal.Subject != params.Account {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(&models.Error{
				Message: "User can manage its own templates only",
			})
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	templates, err := h.handle(params.HTTPRequest.Context(), params.Account)
	if err != nil {
		return InternalError(err, log, false)
	}

	return template_management.NewAccountTemplateListOK().WithPayload(templates)
}
