package handlers

import (
	"context"
	"errors"
	"fmt"
	"net/http"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/rendering"
	op "orus.io/orus-io/rednerd/restapi/operations/template_management"
	"orus.io/orus-io/rednerd/utils"
)

// NewTemplateVarlistHandler creates a TemplateVarlistHandler.
func NewTemplateVarlistHandler(
	db *sqlx.DB,
	templateRegistry *rendering.TemplateEngineRegistry,
) *TemplateVarlistHandler {
	return &TemplateVarlistHandler{db, templateRegistry}
}

// TemplateVarlistHandler handles POST /api/v1/render.
type TemplateVarlistHandler struct {
	db               *sqlx.DB
	templateRegistry *rendering.TemplateEngineRegistry
}

func (h *TemplateVarlistHandler) handle(
	ctx context.Context,
	template *models.Template,
) ([]string, error) {
	if template == nil {
		return nil, utils.HTTPBadRequest(
			errors.New("no template given"))
	}

	log := zerolog.Ctx(ctx)

	if template.Account != "" && template.Name != "" {
		templatedb := models.TemplateDB{}
		sql := database.NewSQLHelper(ctx, h.db, *log)
		if err := sql.GetWhere(
			&templatedb,
			squirrel.Eq{
				models.TemplateDBAccountColumn: template.Account,
				models.TemplateDBNameColumn:    template.Name,
			},
		); err != nil {
			return nil, err
		}
		templatedb.FromDB()
		template = &templatedb.Template
	}
	tmpl, err := h.templateRegistry.Load(ctx, template)
	if err != nil {
		if err == rendering.ErrUnknownTemplateLanguage {
			return nil, utils.HTTPBadRequest(
				fmt.Errorf("unknown template language: %s", template.Language))
		}

		return nil, utils.HTTPBadRequest(
			fmt.Errorf("error loading template: %s", err))
	}
	varlist, err := tmpl.Varlist()
	if err != nil {
		return nil, utils.HTTPBadRequest(
			fmt.Errorf("error listing template variables: %s", err))
	}

	return varlist, nil
}

// Handle the incoming request.
func (h *TemplateVarlistHandler) Handle(params op.TemplateVarlistParams, principals *models.Principal) middleware.Responder {
	varlist, err := h.handle(
		params.HTTPRequest.Context(),
		params.Request,
	)
	if err != nil {
		if he, isHTTPError := err.(utils.HTTPError); isHTTPError {
			r := op.NewTemplateVarlistDefault(he.StatusCode)
			if he.Err != nil {
				r.WithPayload(models.FromError(he.Err))
			}

			return r
		}

		return op.
			NewTemplateVarlistDefault(http.StatusInternalServerError).
			WithPayload(models.FromError(err))
	}

	return op.NewTemplateVarlistOK().WithPayload(varlist)
}
